"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Path = tslib_1.__importStar(require("path"));
const tsm = tslib_1.__importStar(require("ts-morph"));
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const nexus_logger_1 = require("../nexus-logger");
const log = nexus_logger_1.rootLogger.child('add-to-context-extractor');
/**
 * Extract types from all `addToContext` calls.
 */
function extractContextTypes(program) {
    const typeImportsIndex = {};
    const checker = program.getTypeChecker();
    const contextTypeContributions = {
        typeImports: [],
        types: [],
    };
    const appSourceFiles = program.getSourceFiles().filter((sf) => !sf.fileName.match(/node_modules/));
    log.trace('got app source files', {
        count: appSourceFiles.length,
    });
    appSourceFiles.forEach(visit);
    log.trace('finished compiler extension processing', {
        contextTypeContributions,
    });
    // flush deduped type imports
    contextTypeContributions.typeImports.push(...Object.values(typeImportsIndex));
    return contextTypeContributions;
    /**
     * Given a node, traverse the tree of nodes under it.
     */
    function visit(n) {
        // log.trace('visiting node', {
        //   kindNum: n.kind,
        //   kindName: ts.SyntaxKind[n.kind],
        //   text: n.getText(),
        // })
        if (typescript_1.default.isCallExpression(n)) {
            const firstToken = n.getFirstToken();
            const lastToken = n.expression.getLastToken();
            if (firstToken !== undefined &&
                typescript_1.default.isIdentifier(firstToken) &&
                firstToken.text === 'schema' &&
                lastToken !== undefined &&
                typescript_1.default.isIdentifier(lastToken) &&
                lastToken.text === 'addToContext') {
                log.trace('found call', {
                    text: lastToken.getText(),
                });
                // Get the argument passed to addToContext so we can extract its type
                if (n.arguments.length === 0) {
                    log.trace('no args passed to call, the user should see a static type error, stopping extraction');
                    return;
                }
                if (n.arguments.length > 1) {
                    log.trace('multiple args passed to call, the user should see a static type error, stopping extraction');
                    return;
                }
                const contextAdder = n.arguments[0];
                const contextAdderType = checker.getTypeAtLocation(contextAdder);
                const contextAdderSigs = contextAdderType.getCallSignatures();
                log.trace('found call arg', { text: contextAdder.getText() });
                // Get the signature of the argument so we can extract its return type
                if (contextAdderSigs.length === 0) {
                    log.trace('arg had no signatures, this means the user passed a non-callable, the user should see a static type error, stopping context type extraction');
                    return;
                }
                if (contextAdderSigs.length > 1) {
                    log.warn('An overloaded function passed to addToContext. The first signature will be taken. This choice is arbitrary and may result in bad context extraction.', { text: contextAdder.getText() });
                }
                const contextAdderSig = contextAdderSigs[0];
                const contextAdderRetType = checker.getReturnTypeOfSignature(contextAdderSig);
                const unwrappedContextAdderRetType = unwrapMaybePromise(contextAdderRetType, checker);
                const contextAdderRetTypeString = checker.typeToString(unwrappedContextAdderRetType, undefined, typescript_1.default.TypeFormatFlags.NoTruncation);
                contextTypeContributions.types.push(contextAdderRetTypeString);
                // search for named references, they will require importing later on
                const contextAdderRetProps = unwrappedContextAdderRetType.getProperties();
                for (const prop of contextAdderRetProps) {
                    log.trace('processing prop', { name: prop.getName() });
                    const n = prop.declarations[0];
                    const tsmn = tsm.createWrappedNode(n, { typeChecker: checker });
                    const t = tsmn.getType();
                    if (t)
                        if (t.getAliasSymbol()) {
                            log.trace('found alias', {
                                type: t.getText(undefined, typescript_1.default.TypeFormatFlags.NoTruncation),
                            });
                            const info = extractTypeImportInfoFromType(t);
                            if (info) {
                                typeImportsIndex[info.name] = info;
                            }
                        }
                        else if (t.isIntersection()) {
                            log.trace('found intersection', {
                                types: t
                                    .getIntersectionTypes()
                                    .map((t) => t.getText(undefined, typescript_1.default.TypeFormatFlags.NoTruncation)),
                            });
                            const infos = t
                                .getIntersectionTypes()
                                .map((t) => extractTypeImportInfoFromType(t))
                                .filter((info) => info !== null);
                            if (infos.length) {
                                infos.forEach((info) => {
                                    typeImportsIndex[info.name] = info;
                                });
                            }
                        }
                        else if (t.isUnion()) {
                            log.trace('found union', {
                                types: t.getUnionTypes().map((t) => t.getText(undefined, typescript_1.default.TypeFormatFlags.NoTruncation)),
                            });
                            const infos = t
                                .getUnionTypes()
                                .map((t) => extractTypeImportInfoFromType(t))
                                .filter((info) => info !== null);
                            if (infos.length) {
                                infos.forEach((info) => {
                                    typeImportsIndex[info.name] = info;
                                });
                            }
                        }
                        else {
                            const info = extractTypeImportInfoFromType(t);
                            if (info) {
                                typeImportsIndex[info.name] = info;
                            }
                        }
                }
            }
        }
        else {
            n.forEachChild(visit);
        }
    }
}
exports.extractContextTypes = extractContextTypes;
function extractTypeImportInfoFromType(t) {
    var _a;
    let sym = t.getAliasSymbol();
    let name = sym === null || sym === void 0 ? void 0 : sym.getName();
    log.trace('found prop type alias symbol?', { found: !!sym });
    if (t.isArray()) {
        return extractTypeImportInfoFromType(t.getArrayElementTypeOrThrow());
    }
    if (!sym) {
        sym = t.getSymbol();
        log.trace('found prop type symbol?', { found: !!sym });
        if (!sym)
            return null;
        name = sym.getName();
        // not alias but is inline, then skip
        if (name === '__object')
            return null;
        if (name === '__type')
            return null;
    }
    log.trace('found name?', { name });
    if (!name)
        return null;
    const d = (_a = sym.getDeclarations()) === null || _a === void 0 ? void 0 : _a[0];
    if (!d)
        throw new Error('A type with a symbol but the symbol has no declaration');
    const sourceFile = d.getSourceFile();
    const { modulePath, isNode } = getAbsoluteImportPath(sourceFile);
    return {
        name: name,
        modulePath: modulePath,
        isExported: sourceFile.getExportedDeclarations().has(name),
        isNode: isNode,
    };
}
function unwrapMaybePromise(type, checker) {
    var _a;
    if (((_a = type.symbol) === null || _a === void 0 ? void 0 : _a.name) === 'Promise') {
        const typeArgs = type.typeArguments;
        if (typeArgs) {
            const wrappedType = typeArgs[0];
            return wrappedType;
        }
    }
    return type;
}
exports.unwrapMaybePromise = unwrapMaybePromise;
function getAbsoluteImportPath(sourceFile) {
    var _a, _b;
    let isNode = false;
    let modulePath = Path.join(Path.dirname(sourceFile.getFilePath()), sourceFile.getBaseNameWithoutExtension());
    const nodeModule = (_a = modulePath.match(/node_modules\/@types\/node\/(.+)/)) === null || _a === void 0 ? void 0 : _a[1];
    if (nodeModule) {
        modulePath = nodeModule;
        isNode = true;
    }
    else {
        const externalPackage = (_b = modulePath.match(/node_modules\/@types\/(.+)/)) === null || _b === void 0 ? void 0 : _b[1];
        if (externalPackage) {
            modulePath = externalPackage;
        }
    }
    return { isNode, modulePath };
}
//# sourceMappingURL=extractor.js.map