import { PackageJson } from 'type-fest';
import { ParsedCommandLine } from 'typescript';
import * as PackageManager from '../package-manager';
export declare const DEFAULT_BUILD_FOLDER_PATH_RELATIVE_TO_PROJECT_ROOT = "node_modules/.build";
/**
 * The part of layout data resulting from the dynamic file/folder inspection.
 */
export declare type ScanResult = {
    app: {
        exists: true;
        path: string;
    } | {
        exists: false;
        path: null;
    };
    project: {
        name: string;
        isAnonymous: boolean;
    };
    sourceRoot: string;
    projectRoot: string;
    schemaModules: string[];
    tsConfig: {
        content: ParsedCommandLine;
        path: string;
    };
    packageManagerType: PackageManager.PackageManager['type'];
    packageJson: null | {
        dir: string;
        path: string;
        content: PackageJson;
    };
};
/**
 * The combination of manual datums the user can specify about the layout plus
 * the dynamic scan results.
 */
export declare type Data = ScanResult & {
    buildOutput: string;
    startModuleOutPath: string;
    startModuleInPath: string;
};
/**
 * Layout represents the important edges of the project to support things like
 * scaffolding, build, and dev against the correct paths.
 */
export declare type Layout = Data & {
    /**
     * Property that aliases all the and only the data properties, makes it
     * easy to e.g. serialize just the data.
     */
    data: Data;
    projectRelative(filePath: string): string;
    projectPath(...subPaths: string[]): string;
    sourceRelative(filePath: string): string;
    sourcePath(...subPaths: string[]): string;
    packageManager: PackageManager.PackageManager;
};
interface Options {
    /**
     * The place to output the build, relative to project root.
     */
    buildOutput?: string;
    entrypointPath?: string;
    cwd?: string;
}
/**
 * Perform a layout scan and return results with attached helper functions.
 */
export declare function create(options?: Options): Promise<Layout>;
/**
 * Create a layout based on received data. Useful for taking in serialized scan
 * data from another process that would be wasteful to re-calculate.
 */
export declare function createFromData(layoutData: Data): Layout;
/**
 * Analyze the user's project files/folders for how conventions are being used
 * and where key modules exist.
 */
export declare function scan(opts?: {
    cwd?: string;
    entrypointPath?: string;
}): Promise<ScanResult>;
/**
 * Find the (optional) app module in the user's project.
 */
export declare function findAppModule(opts: {
    projectRoot: string;
}): string | null;
/**
 * Find the project root directory. This can be different than the source root
 * directory. For example the classic project structure where there is a root
 * `src` folder. `src` folder would be considered the "source root".
 *
 * Project root is considered to be the first package.json found from cwd upward
 * to disk root. If not package.json is found then cwd is taken to be the
 * project root.
 *
 * todo update jsdoc or make it true again
 *
 */
export declare function findProjectDir(): string;
/**
 * Detect whether or not CWD is inside a nexus project. nexus project is
 * defined as there being a package.json in or above CWD with nexus as a
 * direct dependency.
 */
export declare function scanProjectType(opts: {
    cwd: string;
}): Promise<{
    type: 'unknown' | 'new';
} | {
    type: 'NEXUS_project' | 'node_project';
    packageJson: {};
    packageJsonLocation: {
        path: string;
        dir: string;
    };
}>;
export declare function saveDataForChildProcess(layout: Layout): {
    NEXUS_LAYOUT: string;
};
/**
 * Load the layout data from a serialized version stored in the environment. If
 * it is not found then a warning will be logged and it will be recalculated.
 * For this reason the function is async however under normal circumstances it
 * should be as-if sync.
 */
export declare function loadDataFromParentProcess(): Promise<Layout>;
export {};
//# sourceMappingURL=layout.d.ts.map