"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const common_tags_1 = require("common-tags");
const FS = tslib_1.__importStar(require("fs-jetpack"));
const Path = tslib_1.__importStar(require("path"));
const fs_1 = require("../../lib/fs");
const start_module_1 = require("../../runtime/start/start-module");
const nexus_logger_1 = require("../nexus-logger");
const PackageManager = tslib_1.__importStar(require("../package-manager"));
const process_1 = require("../process");
const Schema = tslib_1.__importStar(require("./schema-modules"));
const tsconfig_1 = require("./tsconfig");
exports.DEFAULT_BUILD_FOLDER_PATH_RELATIVE_TO_PROJECT_ROOT = 'node_modules/.build';
const log = nexus_logger_1.rootLogger.child('layout');
const optionDefaults = {
    buildOutput: exports.DEFAULT_BUILD_FOLDER_PATH_RELATIVE_TO_PROJECT_ROOT,
};
/**
 * Perform a layout scan and return results with attached helper functions.
 */
async function create(options) {
    var _a;
    const cwd = (_a = options === null || options === void 0 ? void 0 : options.cwd) !== null && _a !== void 0 ? _a : process.cwd();
    const normalizedEntrypoint = normalizeEntrypoint(options === null || options === void 0 ? void 0 : options.entrypointPath, cwd);
    // TODO lodash merge defaults or something
    const scanResult = await scan({ cwd, entrypointPath: normalizedEntrypoint });
    const buildOutput = getBuildOutput(options === null || options === void 0 ? void 0 : options.buildOutput, scanResult);
    const outputInfo = {
        buildOutput,
        startModuleInPath: Path.join(scanResult.sourceRoot, start_module_1.START_MODULE_NAME + '.ts'),
        startModuleOutPath: Path.join(buildOutput, start_module_1.START_MODULE_NAME + '.js'),
    };
    log.trace('additional layout data', { data: outputInfo });
    const layout = createFromData(Object.assign(Object.assign({}, scanResult), outputInfo));
    /**
     * Save the created layout in the env
     */
    process.env = Object.assign(Object.assign({}, process.env), saveDataForChildProcess(layout));
    return layout;
}
exports.create = create;
/**
 * Create a layout based on received data. Useful for taking in serialized scan
 * data from another process that would be wasteful to re-calculate.
 */
function createFromData(layoutData) {
    return Object.assign(Object.assign({}, layoutData), { data: layoutData, projectRelative: Path.relative.bind(null, layoutData.projectRoot), sourceRelative: Path.relative.bind(null, layoutData.sourceRoot), sourcePath(...subPaths) {
            return Path.join(layoutData.sourceRoot, ...subPaths);
        },
        projectPath(...subPaths) {
            return Path.join(layoutData.projectRoot, ...subPaths);
        }, packageManager: PackageManager.createPackageManager(layoutData.packageManagerType, {
            projectRoot: layoutData.projectRoot,
        }) });
}
exports.createFromData = createFromData;
/**
 * Analyze the user's project files/folders for how conventions are being used
 * and where key modules exist.
 */
async function scan(opts) {
    var _a, _b;
    log.trace('starting scan');
    const projectRoot = (_a = opts === null || opts === void 0 ? void 0 : opts.cwd) !== null && _a !== void 0 ? _a : process.cwd();
    const packageManagerType = await PackageManager.detectProjectPackageManager({ projectRoot });
    const maybePackageJson = findPackageJson({ projectRoot });
    const maybeAppModule = (_b = opts === null || opts === void 0 ? void 0 : opts.entrypointPath) !== null && _b !== void 0 ? _b : findAppModule({ projectRoot });
    let maybeSchemaModules = Schema.findDirOrModules({ projectRoot });
    const tsConfig = await tsconfig_1.readOrScaffoldTsconfig({
        projectRoot,
    });
    // Remove entrypoint from schema modules (can happen if the entrypoint is named graphql.ts or inside a graphql/ folder)
    if (maybeSchemaModules && maybeAppModule && maybeSchemaModules.includes(maybeAppModule)) {
        maybeSchemaModules = maybeSchemaModules.filter((s) => s !== maybeAppModule);
    }
    const result = {
        app: maybeAppModule === null
            ? { exists: false, path: maybeAppModule }
            : { exists: true, path: maybeAppModule },
        projectRoot,
        sourceRoot: tsConfig.content.options.rootDir,
        schemaModules: maybeSchemaModules,
        project: readProjectInfo(opts),
        tsConfig,
        packageManagerType,
        packageJson: maybePackageJson,
    };
    log.trace('completed scan', { result });
    if (result.app.exists === false && result.schemaModules.length === 0) {
        log.error(checks.no_app_or_schema_modules.explanations.problem);
        log.error(checks.no_app_or_schema_modules.explanations.solution);
        process.exit(1);
    }
    return result;
}
exports.scan = scan;
// todo allow user to configure these for their project
const CONVENTIONAL_ENTRYPOINT_MODULE_NAME = 'app';
const CONVENTIONAL_ENTRYPOINT_FILE_NAME = `${CONVENTIONAL_ENTRYPOINT_MODULE_NAME}.ts`;
const checks = {
    no_app_or_schema_modules: {
        code: 'no_app_or_schema_modules',
        // prettier-ignore
        explanations: {
            problem: `We could not find any ${Schema.MODULE_NAME} modules or app entrypoint`,
            solution: common_tags_1.stripIndent `
        Please do one of the following:

          1. Create a (${chalk_1.default.yellow(Schema.CONVENTIONAL_SCHEMA_FILE_NAME)} file and write your GraphQL type definitions in it.
          2. Create a ${chalk_1.default.yellow(Schema.DIR_NAME)} directory and write your GraphQL type definitions inside files there.
          3. Create an ${chalk_1.default.yellow(CONVENTIONAL_ENTRYPOINT_FILE_NAME)} file.
    `,
        }
    },
};
/**
 * Find the (optional) app module in the user's project.
 */
function findAppModule(opts) {
    log.trace('looking for app module');
    const path = fs_1.findFile(CONVENTIONAL_ENTRYPOINT_FILE_NAME, opts);
    log.trace('done looking for app module', { path });
    return path;
}
exports.findAppModule = findAppModule;
/**
 * Find the project root directory. This can be different than the source root
 * directory. For example the classic project structure where there is a root
 * `src` folder. `src` folder would be considered the "source root".
 *
 * Project root is considered to be the first package.json found from cwd upward
 * to disk root. If not package.json is found then cwd is taken to be the
 * project root.
 *
 * todo update jsdoc or make it true again
 *
 */
function findProjectDir() {
    return process.cwd();
}
exports.findProjectDir = findProjectDir;
/**
 * Detect whether or not CWD is inside a nexus project. nexus project is
 * defined as there being a package.json in or above CWD with nexus as a
 * direct dependency.
 */
async function scanProjectType(opts) {
    var _a;
    const packageJsonLocation = findPackageJsonRecursivelyUpward(opts);
    if (packageJsonLocation === null) {
        if (await isEmptyCWD()) {
            return { type: 'new' };
        }
        return { type: 'unknown' };
    }
    const packageJson = FS.read(packageJsonLocation.path, 'json');
    if ((_a = packageJson === null || packageJson === void 0 ? void 0 : packageJson.dependencies) === null || _a === void 0 ? void 0 : _a['nexus']) {
        return {
            type: 'NEXUS_project',
            packageJson: packageJsonLocation,
            packageJsonLocation: packageJsonLocation,
        };
    }
    return {
        type: 'node_project',
        packageJson: packageJsonLocation,
        packageJsonLocation: packageJsonLocation,
    };
}
exports.scanProjectType = scanProjectType;
/**
 * Check if the CWD is empty of any files or folders.
 * TODO we should make nice exceptions for known meaningless files, like .DS_Store
 */
async function isEmptyCWD() {
    const contents = await FS.listAsync();
    return contents === undefined || contents.length === 0;
}
const ENV_VAR_DATA_NAME = 'NEXUS_LAYOUT';
function saveDataForChildProcess(layout) {
    return {
        [ENV_VAR_DATA_NAME]: JSON.stringify(layout.data),
    };
}
exports.saveDataForChildProcess = saveDataForChildProcess;
/**
 * Load the layout data from a serialized version stored in the environment. If
 * it is not found then a warning will be logged and it will be recalculated.
 * For this reason the function is async however under normal circumstances it
 * should be as-if sync.
 */
async function loadDataFromParentProcess() {
    const savedData = process.env[ENV_VAR_DATA_NAME];
    if (!savedData) {
        log.trace('WARNING an attempt to load saved layout data was made but no serialized data was found in the environment. This may represent a bug. Layout is being re-calculated as a fallback solution. This should result in the same layout data (if not, another probably bug, compounding confusion) but at least adds latentency to user experience.');
        return create({}); // todo no build output...
    }
    else {
        return createFromData(JSON.parse(savedData));
    }
}
exports.loadDataFromParentProcess = loadDataFromParentProcess;
function readProjectInfo(opts) {
    var _a;
    const localFS = FS.cwd((_a = opts === null || opts === void 0 ? void 0 : opts.cwd) !== null && _a !== void 0 ? _a : process.cwd());
    try {
        const packageJson = require(localFS.path('package.json'));
        if (packageJson.name) {
            return {
                name: packageJson.name,
                isAnonymous: false,
            };
        }
    }
    catch (_b) { }
    return {
        name: 'anonymous',
        isAnonymous: true,
    };
}
/**
 * Find the package.json file path. Looks recursively upward to disk root.
 * Starts looking in CWD If no package.json found along search, returns null.
 */
function findPackageJsonRecursivelyUpward(opts) {
    return fs_1.findFileRecurisvelyUpwardSync('package.json', opts);
}
/**
 *
 */
function findPackageJson(opts) {
    const packageJsonPath = FS.path(opts.projectRoot, 'package.json');
    try {
        const content = FS.read(packageJsonPath, 'json');
        return {
            content,
            path: packageJsonPath,
            dir: Path.dirname(packageJsonPath),
        };
    }
    catch (_a) {
        return null;
    }
}
function normalizeEntrypoint(entrypoint, cwd) {
    if (!entrypoint) {
        return undefined;
    }
    const absoluteEntrypoint = entrypoint.startsWith('/') ? entrypoint : Path.join(cwd, entrypoint);
    if (!absoluteEntrypoint.endsWith('.ts')) {
        process_1.fatal('Entrypoint must be a .ts file', { path: absoluteEntrypoint });
    }
    if (!FS.exists(absoluteEntrypoint)) {
        process_1.fatal('Entrypoint does not exist', { path: absoluteEntrypoint });
    }
    return absoluteEntrypoint;
}
/**
 * Get the relative output build path
 * Precedence: User's input > tsconfig.json's outDir > default
 */
function getBuildOutput(buildOutput, scanResult) {
    var _a;
    const output = (_a = buildOutput !== null && buildOutput !== void 0 ? buildOutput : scanResult.tsConfig.content.options.outDir) !== null && _a !== void 0 ? _a : optionDefaults.buildOutput;
    if (Path.isAbsolute(output)) {
        return output;
    }
    return Path.join(scanResult.projectRoot, output);
}
//# sourceMappingURL=layout.js.map