export declare type MaybePromise<T = void> = T | Promise<T>;
export declare type CallbackRegistrer<F> = (f: F) => void;
export declare type SideEffector = () => MaybePromise;
export declare type Param1<F> = F extends (p: infer P, ...args: any[]) => any ? P : never;
/**
 * DeepPartial - borrowed from `utility-types`
 * @desc Partial that works for deeply nested structure
 * @example
 *   Expect: {
 *     first?: {
 *       second?: {
 *         name?: string;
 *       };
 *     };
 *   }
 *   type NestedProps = {
 *     first: {
 *       second: {
 *         name: string;
 *       };
 *     };
 *   };
 *   type PartialNestedProps = DeepPartial<NestedProps>;
 */
export declare type DeepPartial<T> = T extends Function ? T : T extends Array<infer U> ? DeepPartialArray<U> : T extends object ? DeepPartialObject<T> : T | undefined;
/** @private */
export interface DeepPartialArray<T> extends Array<DeepPartial<T>> {
}
/** @private */
export declare type DeepPartialObject<T> = {
    [P in keyof T]?: DeepPartial<T[P]>;
};
/**
 * DeepRequired - borrowed from `utility-types`
 * @desc Required that works for deeply nested structure
 * @example
 *   Expect: {
 *     first: {
 *       second: {
 *         name: string;
 *       };
 *     };
 *   }
 *   type NestedProps = {
 *     first?: {
 *       second?: {
 *         name?: string;
 *       };
 *     };
 *   };
 *   type RequiredNestedProps = DeepRequired<NestedProps>;
 */
export declare type DeepRequired<T> = T extends (...args: any[]) => any ? T : T extends any[] ? DeepRequiredArray<T[number]> : T extends object ? DeepRequiredObject<T> : T;
/** @private */
export interface DeepRequiredArray<T> extends Array<DeepRequired<NonUndefined<T>>> {
}
/** @private */
export declare type DeepRequiredObject<T> = {
    [P in keyof T]-?: DeepRequired<NonUndefined<T[P]>>;
};
export declare type NonUndefined<A> = A extends undefined ? never : A;
/**
 * Guarantee the length of a given string, padding before or after with the
 * given character. If the given string is longer than  the span target, then it
 * will be cropped.
 */
export declare function span(padSide: 'padBefore' | 'padAfter', padChar: string, target: number, content: string): string;
/**
 * Guarantee the length of a given string, padding with space as needed. Content
 * is aligned left and if exceeds span target length to begin with gets cropped.
 */
export declare const clampSpace: (target: number, content: string) => string;
/**
 * Create a string of space of the given length.
 */
export declare function spanSpace(num: number): string;
/**
 * Create a string of the given length and character
 */
export declare function spanChar(num: number, char: string): string;
/**
 * Guarantee the length of a given string, padding with space as needed. Content
 * is aligned right and if exceeds span target length to begin with gets cropped.
 */
export declare const spanSpaceRight: (target: number, content: string) => string;
/**
 * Use this to make assertion at end of if-else chain that all members of a
 * union have been accounted for.
 */
export declare function casesHandled(x: never): never;
/**
 * Create a function that will only ever return the given value when called.
 */
export declare function constant<T>(x: T): () => T;
/**
 * Create a range of integers.
 */
export declare function range(times: number): number[];
export declare type OmitFirstArg<Func> = Func extends (firstArg: any, ...args: infer Args) => infer Ret ? (...args: Args) => Ret : never;
/**
 * Generate a random project name.
 */
export declare function generateProjectName(): string;
/**
 * Get the name of the CWD or if at disk root and thus making it impossible to
 * extract a meaningful name, generate one.
 */
export declare function CWDProjectNameOrGenerate(opts?: {
    cwd: string;
}): string;
/**
 * Creates a new git repository with an initial commit of all contents at the
 * time this function is run.
 */
export declare function createGitRepository(): Promise<void>;
export declare function requireModule(config: {
    depName: string;
    optional: boolean;
}): null | unknown;
/**
 * Check whether Worker Threads are available. In Node 10, workers aren't available by default.
 */
export declare function areWorkerThreadsAvailable(): boolean;
export declare function repalceInObject<C extends object>(dynamicPattern: string | RegExp, replacement: string, content: C): C;
export declare function replaceEvery(str: string, dynamicPattern: string, replacement: string): string;
/**
 * Creates an array of elements split into two groups.
 * The first of which contains elements predicate returns truthy for, the second of which contains elements predicate returns falsey for.
 * The predicate is invoked with one argument: (value).
 */
export declare function partition<T>(array: Array<T>, predicate: (value: T) => boolean): [Array<T>, Array<T>];
/**
 * Render IPv6 `::` as localhost. By default Node servers will use :: if IPv6
 * host is available otherwise IPv4 0.0.0.0. In local development it seems that
 * rendering as localhost makes the most sense as to what the user expects.
 * According to Node docs most operating systems that are supporting IPv6
 * somehow bind `::` to `0.0.0.0` anyways.
 */
export declare function prettifyHost(host: string): string;
/**
 * Makes sure, that there is only one execution at a time
 * and the last invocation doesn't get lost (tail behavior of debounce)
 * Mostly designed for watch mode
 */
export declare function simpleDebounce<T extends (...args: any[]) => Promise<any>>(fn: T): T;
//# sourceMappingURL=index.d.ts.map