"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Logger = tslib_1.__importStar(require("@nexus/logger"));
const common_tags_1 = require("common-tags");
const Plugin = tslib_1.__importStar(require("../lib/plugin"));
const Reflection = tslib_1.__importStar(require("../lib/reflection"));
const Schema = tslib_1.__importStar(require("./schema"));
const Server = tslib_1.__importStar(require("./server"));
const Settings = tslib_1.__importStar(require("./settings"));
const log = Logger.create({ name: 'app' });
/**
 * Create new app state. Be careful to pass this state to components to complete its
 * data. The data returned only contains core state, despite what the return
 * type says.
 */
function createAppState() {
    const appState = {
        assembled: false,
        running: false,
        plugins: [],
    };
    return appState;
}
exports.createAppState = createAppState;
/**
 * Create an app instance
 */
function create() {
    const appState = createAppState();
    const serverComponent = Server.create(appState);
    const schemaComponent = Schema.create(appState);
    const settingsComponent = Settings.create({
        serverSettings: serverComponent.private.settings,
        schemaSettings: schemaComponent.private.settings,
        log,
    });
    const api = {
        log: log,
        settings: settingsComponent.public,
        schema: schemaComponent.public,
        server: serverComponent.public,
        // todo call this in the start module
        assemble() {
            if (appState.assembled)
                return;
            appState.assembled = true;
            if (Reflection.isReflectionStage('plugin'))
                return;
            const loadedRuntimePlugins = Plugin.importAndLoadRuntimePlugins(appState.plugins);
            schemaComponent.private.assemble(loadedRuntimePlugins);
            if (Reflection.isReflectionStage('typegen'))
                return;
            schemaComponent.private.checks();
            serverComponent.private.assemble(loadedRuntimePlugins, appState.schemaComponent.schema);
        },
        async start() {
            if (Reflection.isReflection())
                return;
            if (appState.running)
                return;
            await serverComponent.private.start();
            appState.running = true;
        },
        async stop() {
            if (Reflection.isReflection())
                return;
            if (!appState.running)
                return;
            await serverComponent.private.stop();
            appState.running = false;
        },
        use(plugin) {
            if (appState.assembled === true) {
                log.warn(common_tags_1.stripIndent `
          A plugin was ignored because it was loaded after the server was started
          Make sure to call \`use\` before you call \`server.start\`
        `);
            }
            appState.plugins.push(plugin);
        },
        private: {
            state: appState,
        },
    };
    return api;
}
exports.create = create;
//# sourceMappingURL=app.js.map