"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const express_1 = tslib_1.__importDefault(require("express"));
const ExpressGraphQL = tslib_1.__importStar(require("express-graphql"));
const HTTP = tslib_1.__importStar(require("http"));
const strip_ansi_1 = tslib_1.__importDefault(require("strip-ansi"));
const DevMode = tslib_1.__importStar(require("../dev-mode"));
const logger_1 = require("./logger");
const ServerSettings = tslib_1.__importStar(require("./settings"));
// Avoid forcing users to use esModuleInterop
const createExpressGraphql = ExpressGraphQL.default;
const resolverLogger = logger_1.log.child('graphql');
function setupExpress(express, settings) {
    const http = HTTP.createServer();
    http.on('request', express);
    express.use(settings.path, createExpressGraphql((req) => {
        return settings.context(req).then((resolvedCtx) => (Object.assign(Object.assign({}, settings), { context: resolvedCtx, customFormatErrorFn: (error) => {
                var _a;
                const colorlessMessage = strip_ansi_1.default(error.message);
                if (process.env.NEXUS_STAGE === 'dev') {
                    resolverLogger.error((_a = error.stack) !== null && _a !== void 0 ? _a : error.message);
                }
                else {
                    resolverLogger.error('An exception occured in one of your resolver', {
                        error: error.stack ? strip_ansi_1.default(error.stack) : colorlessMessage,
                    });
                }
                error.message = colorlessMessage;
                return error;
            } })));
    }));
    if (settings.playground) {
        express.get(settings.playground.path, (_req, res) => {
            res.send(`
        <!DOCTYPE html>
        <html>

        <head>
          <meta charset=utf-8/>
          <meta name="viewport" content="user-scalable=no, initial-scale=1.0, minimum-scale=1.0, maximum-scale=1.0, minimal-ui">
          <title>GraphQL Playground</title>
          <link rel="stylesheet" href="//cdn.jsdelivr.net/npm/graphql-playground-react/build/static/css/index.css" />
          <link rel="shortcut icon" href="//cdn.jsdelivr.net/npm/graphql-playground-react/build/favicon.png" />
          <script src="//cdn.jsdelivr.net/npm/graphql-playground-react/build/static/js/middleware.js"></script>
        </head>

        <body>
          <div id="root">
            <style>
              body {
                background-color: rgb(23, 42, 58);
                font-family: Open Sans, sans-serif;
                height: 90vh;
              }

              #root {
                height: 100%;
                width: 100%;
                display: flex;
                align-items: center;
                justify-content: center;
              }

              .loading {
                font-size: 32px;
                font-weight: 200;
                color: rgba(255, 255, 255, .6);
                margin-left: 20px;
              }

              img {
                width: 78px;
                height: 78px;
              }

              .title {
                font-weight: 400;
              }
            </style>
            <img src='//cdn.jsdelivr.net/npm/graphql-playground-react/build/logo.png' alt=''>
            <div class="loading"> Loading
              <span class="title">GraphQL Playground</span>
            </div>
          </div>
          <script>window.addEventListener('load', function (event) {
              GraphQLPlayground.init(document.getElementById('root'), {
                endpoint: '${settings.path}'
              })
            })</script>
        </body>

        </html>
      `);
        });
    }
    return {
        start: () => new Promise((res) => {
            http.listen({ port: settings.port, host: settings.host }, () => {
                // - We do not support listening on unix domain sockets so string
                //   value will never be present here.
                // - We are working within the listen callback so address will not be null
                const address = http.address();
                settings.startMessage({
                    port: address.port,
                    host: address.address,
                    ip: address.address,
                    path: settings.path,
                    playgroundPath: settings.playground ? settings.playground.path : undefined,
                });
                res();
            });
        }),
        stop: () => new Promise((res, rej) => {
            http.close((err) => {
                if (err) {
                    rej(err);
                }
                else {
                    res();
                }
            });
        }),
    };
}
function create() {
    const express = express_1.default();
    let server = null;
    const state = {
        settings: ServerSettings.defaultSettings(),
    };
    return {
        express,
        async setupAndStart(opts) {
            const context = contextFactory(opts.contextContributors, opts.plugins);
            server = setupExpress(express, Object.assign(Object.assign({}, state.settings), { schema: opts.schema, context }));
            await server.start();
            DevMode.sendServerReadySignalToDevModeMaster();
        },
        stop() {
            if (!server) {
                logger_1.log.warn('You called `server.stop` before calling `server.start`');
                return Promise.resolve();
            }
            return server.stop();
        },
        settings: {
            change(newSettings) {
                state.settings = ServerSettings.changeSettings(state.settings, newSettings);
            },
            data: state.settings,
        },
    };
}
exports.create = create;
function contextFactory(contextContributors, plugins) {
    const createContext = async (req) => {
        let context = {};
        // Integrate context from plugins
        for (const plugin of plugins) {
            if (!plugin.context)
                continue;
            const contextContribution = await plugin.context.create(req);
            Object.assign(context, contextContribution);
        }
        // Integrate context from app context api
        // TODO good runtime feedback to user if something goes wrong
        for (const contextContributor of contextContributors) {
            const contextContribution = await contextContributor(req);
            Object.assign(context, contextContribution);
        }
        Object.assign(context, { log: logger_1.log.child('request') });
        return context;
    };
    return createContext;
}
//# sourceMappingURL=server-old.js.map