"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const express_1 = tslib_1.__importDefault(require("express"));
const ExpressGraphQL = tslib_1.__importStar(require("express-graphql"));
const HTTP = tslib_1.__importStar(require("http"));
const strip_ansi_1 = tslib_1.__importDefault(require("strip-ansi"));
const logger_1 = require("./logger");
const request_handler_playground_1 = require("./request-handler-playground");
const settings_1 = require("./settings");
// Avoid forcing users to use esModuleInterop
const createExpressGraphql = ExpressGraphQL.default;
function create(appState) {
    const resolverLogger = logger_1.log.child('graphql');
    const settings = settings_1.createServerSettingsManager();
    const express = express_1.default();
    const state = {
        running: false,
        httpServer: HTTP.createServer(),
    };
    return {
        private: {
            settings,
            state,
            assemble(loadedRuntimePlugins, schema) {
                state.httpServer.on('request', express);
                if (settings.data.playground) {
                    express.get(settings.data.playground.path, request_handler_playground_1.createRequestHandlerPlayground({ graphqlEndpoint: settings.data.path }));
                }
                const createContext = createContextCreator(appState.schemaComponent.contextContributors, loadedRuntimePlugins);
                express.use(settings.data.path, createExpressGraphql((req) => {
                    return createContext(req).then((context) => (Object.assign(Object.assign({}, settings), { context: context, schema: schema, customFormatErrorFn: (error) => {
                            var _a;
                            const colorlessMessage = strip_ansi_1.default(error.message);
                            if (process.env.NEXUS_STAGE === 'dev') {
                                resolverLogger.error((_a = error.stack) !== null && _a !== void 0 ? _a : error.message);
                            }
                            else {
                                resolverLogger.error('An exception occured in one of your resolver', {
                                    error: error.stack ? strip_ansi_1.default(error.stack) : colorlessMessage,
                                });
                            }
                            error.message = colorlessMessage;
                            return error;
                        } })));
                }));
            },
            async start() {
                await httpListen(state.httpServer, { port: settings.data.port, host: settings.data.host });
                // About !
                // 1. We do not support listening on unix domain sockets so string
                //    value will never be present here.
                // 2. We are working within the listen callback so address will not be null
                const address = state.httpServer.address();
                settings.data.startMessage({
                    port: address.port,
                    host: address.address,
                    ip: address.address,
                    path: settings.data.path,
                    playgroundPath: settings.data.playground ? settings.data.playground.path : undefined,
                });
            },
            async stop() {
                if (!state.running) {
                    logger_1.log.warn('You called `server.stop` but the server was not running.');
                    return Promise.resolve();
                }
                await httpClose(state.httpServer);
                state.running = false;
            },
        },
        public: {
            express,
        },
    };
}
exports.create = create;
/**
 * Combine all the context contributions defined in the app and in plugins.
 */
function createContextCreator(contextContributors, plugins) {
    const createContext = async (req) => {
        let context = {};
        // Integrate context from plugins
        for (const plugin of plugins) {
            if (!plugin.context)
                continue;
            const contextContribution = await plugin.context.create(req);
            Object.assign(context, contextContribution);
        }
        // Integrate context from app context api
        // TODO good runtime feedback to user if something goes wrong
        for (const contextContributor of contextContributors) {
            const contextContribution = await contextContributor(req);
            Object.assign(context, contextContribution);
        }
        Object.assign(context, { log: logger_1.log.child('request') });
        return context;
    };
    return createContext;
}
function httpListen(server, options) {
    return new Promise((res, rej) => {
        server.listen(options, () => {
            res();
        });
    });
}
function httpClose(server) {
    return new Promise((res, rej) => {
        server.close((err) => {
            if (err) {
                rej(err);
            }
            else {
                res();
            }
        });
    });
}
//# sourceMappingURL=server.js.map