"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const get_port_1 = tslib_1.__importDefault(require("get-port"));
const Lo = tslib_1.__importStar(require("lodash"));
const graphql_client_1 = require("../lib/graphql-client");
const Layout = tslib_1.__importStar(require("../lib/layout"));
const Plugin = tslib_1.__importStar(require("../lib/plugin"));
const start_1 = require("./start");
function createAppClient(apiUrl) {
    const client = new graphql_client_1.GraphQLClient(apiUrl);
    return {
        query(queryString, variables) {
            return client.request(queryString, variables);
        },
    };
}
exports.createAppClient = createAppClient;
/**
 * Setup a test context providing utilities to query against your GraphQL API
 *
 * @example
 *
 * With jest
 * ```
 * import { setupTest, TestContext } from 'nexus/testing'
 *
 * let testCtx: TestContext
 *
 * beforeAll(async () => {
 *  testCtx = await setupTest()
 *  await testCtx.server.start()
 * })
 *
 * afterAll(async () => {
 *  await testCtx.server.stop()
 * })
 * ```
 */
async function createTestContext() {
    // Guarantee that development mode features are on
    process.env.NEXUS_STAGE = 'dev';
    // todo figure out some caching system here, e.g. imagine jest --watch mode
    const layout = await Layout.create();
    const pluginManifests = await Plugin.getUsedPlugins(layout);
    const randomPort = await get_port_1.default({ port: get_port_1.default.makeRange(4000, 6000) });
    const app = require('../index');
    app.settings.change({
        server: {
            port: randomPort,
            playground: false,
            startMessage() { },
        },
    });
    const appRunner = await start_1.createDevAppRunner(layout, app);
    const apiUrl = `http://localhost:${appRunner.port}/graphql`;
    const appClient = createAppClient(apiUrl);
    const testContextCore = {
        app: {
            query: appClient.query,
            server: {
                start: appRunner.start,
                stop: appRunner.stop,
            },
        },
    };
    const testContextContributions = Plugin.importAndLoadTesttimePlugins(pluginManifests);
    for (const testContextContribution of testContextContributions) {
        Lo.merge(testContextCore, testContextContribution);
    }
    return testContextCore;
}
exports.createTestContext = createTestContext;
//# sourceMappingURL=testing.js.map